package jpatest.test;

import java.util.Set;

import javax.persistence.EntityManager;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;
import javax.validation.ValidatorFactory;

import jpatest.eao.BaseEAO;
import jpatest.entity.Address;

import org.junit.Test;

/**
 * Test Bean Validation on the Address entity
 * 
 */
public class BeanValidationTest extends MyJpaTestCase {

	@Test
	public void testAddress() {

		Address ad = new Address();
		ad.setStreet("2920 Main Street");
		ad.setCity("LocalTown");
		ad.setState("MI");
		ad.setZip("18081");

		EntityManager em = BaseEAO.createEntityManager();

		try {
			em.getTransaction().begin(); // start transaction
			em.persist(ad); // insert vehicle v into database
			em.getTransaction().commit(); // commit transaction
		} catch (Throwable t) {
			t.printStackTrace();
			em.getTransaction().rollback(); // rollback transaction

			// get bean validation messages
			if (t instanceof ConstraintViolationException) {
				ConstraintViolationException cve = (ConstraintViolationException) t;
				Set<ConstraintViolation<?>> cvSet = cve
						.getConstraintViolations();
				for (ConstraintViolation cv : cvSet) {
					System.out.println("bean validation error: "
							+ cv.getMessage());
				}

			}
		}

		Address add2 = em.find(Address.class, ad.getAddressId());
		assertTrue(add2 != null);

		// delete the newly inserted entity
		try {
			em.getTransaction().begin();
			em.remove(ad);
			em.getTransaction().commit();
		} catch (Throwable t) {
			t.printStackTrace();
			em.getTransaction().rollback();
		}

		em.close(); // close entity manager
	}

	@Test
	public void testBeanValidationAlone() {
		System.out.println("running  testBeanValidationAlone()");

		Address addr = new Address();
		addr.setStreet("2920 Main Street");
		addr.setCity("LocalTown");
		// addr.setState("MI");
		addr.setZip("1808");

		ValidatorFactory factory = Validation.buildDefaultValidatorFactory();
		Validator validator = factory.getValidator();

		System.out.println("validating address. It does not throw exceptions.");
		Set<ConstraintViolation<Address>> cvSet = validator.validate(addr);

		for (ConstraintViolation<Address> cv : cvSet) {
			System.out.format("bean validation error -- %s: %s%n",cv.getPropertyPath(), cv.getMessage());
		}

	}

}
