package jpatest.ejb;

import java.util.List;
import java.util.Map;
import javax.ejb.Stateful;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.PersistenceContextType;
import javax.persistence.SynchronizationType;

import jpatest.entity.Book;
import jpatest.entity.Customer;
import jpatest.entity.LineItem;
import jpatest.entity.Order;

@Stateful
public class ShoppingCartSF {

	@PersistenceContext(type = PersistenceContextType.EXTENDED, synchronization = SynchronizationType.UNSYNCHRONIZED, unitName = "jpaTestJtaPU")
	EntityManager em;
	
	Customer customer;
	Order order;

	// construct a shopping chart for a specified customer
	public void constructCart(Integer custId) {
		customer = em.find(Customer.class, custId);
		order = new Order();
	}

	// keep adding book items to the shopping cart
	public void addToCart(Book book, int quantity) {
		LineItem item = new LineItem();
		item.setBook(book);
		item.setQuantity(quantity);
		List<LineItem> items = order.getLineItems();
		items.add(item);
		order.setLineItems(items);
	}

	// at the end, commit the book items in the shopping cart
	public void confirmOrder() {
		em.joinTransaction();
		
		Map<Integer, Order> orders = customer.getOrders();
		orders.put(customer.getCustomerId(), order);	
		customer.setOrders(orders);
		em.persist(order);
	}
}
