package jpatest.ejb;

import java.util.List;
import java.util.Properties;
import javax.ejb.embeddable.EJBContainer;
import javax.naming.NamingException;

import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.assertTrue;

import jpatest.Vehicle;

/**
 * Test vehicle stateless session bean: VehicleSL, using the Apache OpenEJB
 * embedded container (introduced in EJB 3.1 specification).
 * 
 */
public class VehicleTest {
	
	private EJBContainer ejbContainer;

	@Before
	public void init() {

		Properties env = new Properties();
		env.put("java.naming.factory.initial",
				"org.apache.openejb.client.LocalInitialContextFactory");

		// specify OpenEJB configuration file openejb.xml to load
		env.put("openejb.configuration",
				"C:\\Projects\\jpatestChapter1EJB\\src\\main\\resources\\openejb.xml");

		// To connect to a remote OpenEJB container
		// env.put("java.naming.factory.initial","org.openejb.client.RemoteInitialContextFactory");
		// env.put( "java.naming.provider.url", "localhost:4201" );
		// env.put( "java.naming.security.principal", "user" );
		// env.put( "java.naming.security.credentials", "password" );

		// We want OpenEJB to log everything in debug mode
		env.put("log4j.category.OpenEJB.options", "debug");
		env.put("log4j.category.OpenEJB.startup", "debug");
		env.put("log4j.category.OpenEJB.startup.config", "debug");

		// We tell OpenEJB to scan the whole classpath
		env.put("openejb.deployments.classpath.include", ".*");
		env.put("openejb.deployments.classpath.exclude", "");

		// create embedded EJB container
		ejbContainer = EJBContainer.createEJBContainer(env);
	}

	@Test
	public void testVehicleEJB() throws NamingException {

		VehicleSL vehEJB = (VehicleSL) ejbContainer.getContext().lookup(
		// check console output for the JNDI name for the deployed EJB
				"java:global/jpatestChapter1EJB/VehicleSL");

		String vinPrimaryKey = "4B7HF16Y7SS244328";
		Vehicle v = new Vehicle(vinPrimaryKey, "Ford", "Flex", 2009);

		// insert the new Vehicle object into database
		vehEJB.createVehicle(v);
		System.out.println("initial version = " + v.getVersion());

		// do some update, then version should be incremented by 1
		v.setMake("Mercury");
		Vehicle v2 = vehEJB.updateVehicle(v);
		assertTrue(v2.getVersion() >= v.getVersion() + 1);
		System.out.println("new version = " + v2.getVersion());

		// retrieve all vehicles with make "Mercury"
		List<Vehicle> vehicles = vehEJB.retrieveVehicles("Mercury");
		for (Vehicle h : vehicles) {
			System.out.println("vehicle retrieved: " + h.getVin());
		}

		// delete newly inserted/updated vehicle. Must pass in v2 as a parameter
		// instead of v. Otherwise, OptimisticLockException would be thrown.
		vehEJB.deleteVehicle(v2);
	}

}
